@@ -113,6 +113,7 @@ int rand_get(uint8_t *rand, unsigned int len)
 	return len;
 }
 
+/* Subscriber with 2G-only (COMP128v1) authentication data */
 static void test_gen_vectors_2g_only(void)
 {
 	struct osmo_sub_auth_data aud2g;
@@ -174,6 +175,8 @@ static void test_gen_vectors_2g_only(void)
 	comment_end();
 }
 
+/* Subscriber with separate 2G (COMP128v1) and 3G (MILENAGE) authentication data,
+ * reflects the default configuration of sysmoUSIM-SJS1 */
 static void test_gen_vectors_2g_plus_3g(void)
 {
 	struct osmo_sub_auth_data aud2g;
@@ -284,6 +287,9 @@ void _test_gen_vectors_3g_only__expect_vecs(struct osmo_auth_vector vecs[3])
 	      );
 }
 
+/* Subscriber with only 3G (MILENAGE) authentication data,
+ * reflects the default configuration of sysmoISIM-SJA2. Resulting
+ * tuples are suitable for both 2G and 3G authentication */
 static void test_gen_vectors_3g_only(void)
 {
 	struct osmo_sub_auth_data aud2g;
@@ -454,7 +460,56 @@ static void test_gen_vectors_3g_only(void)
 	comment_end();
 }
 
-void test_gen_vectors_bad_args()
+/* Subscriber with only 3G (XOR) authentication data,
+ * reflects the default configuration of sysmoTSIM-SJAx as well
+ * as many "Test USIM" cards. Resulting tuples are suitable for both
+ * 2G and 3G authentication */
+static void test_gen_vectors_3g_xor(void)
+{
+	struct osmo_sub_auth_data aud2g;
+	struct osmo_sub_auth_data aud3g;
+	struct osmo_auth_vector vec;
+	int rc;
+
+	comment_start();
+
+	aud2g = (struct osmo_sub_auth_data){ 0 };
+
+	aud3g = (struct osmo_sub_auth_data){
+		.type = OSMO_AUTH_TYPE_UMTS,
+		.algo = OSMO_AUTH_ALG_XOR,
+		.u.umts.sqn = 0,
+	};
+
+	osmo_hexparse("000102030405060708090a0b0c0d0e0f",
+		      aud3g.u.umts.k, sizeof(aud3g.u.umts.k));
+	osmo_hexparse("00000000000000000000000000000000",
+		      aud3g.u.umts.opc, sizeof(aud3g.u.umts.opc));
+	next_rand("b5039c57e4a75051551d1a390a71ce48", true);
+
+	vec = (struct osmo_auth_vector){ {0} };
+	VERBOSE_ASSERT(aud3g.u.umts.sqn, == 0, "%"PRIu64);
+	rc = auc_compute_vectors(&vec, 1, &aud2g, &aud3g, NULL, NULL);
+	VERBOSE_ASSERT(rc, == 1, "%d");
+	VERBOSE_ASSERT(aud3g.u.umts.sqn, == 0, "%"PRIu64);
+
+	VEC_IS(&vec,
+	       "  rand: b5039c57e4a75051551d1a390a71ce48\n"
+	       "  autn: 54e0a256565d0000b5029e54e0a25656\n"
+	       "  ck: 029e54e0a256565d141032067cc047b5\n"
+	       "  ik: 9e54e0a256565d141032067cc047b502\n"
+	       "  res: b5029e54e0a256565d141032067cc047\n"
+	       "  res_len: 10\n"
+	       "  kc: 98e880384887f9fe\n"
+	       "  sres: 0ec81877\n"
+	       "  auth_types: 03000000\n"
+	      );
+
+	comment_end();
+}
+
+/* Test a variety of invalid authentication data combinations */
+void test_gen_vectors_bad_args(void)
 {
 	struct osmo_auth_vector vec;
 	uint8_t auts[14];
@@ -613,15 +668,20 @@ int main(int argc, char **argv)
 	void *tall_ctx = talloc_named_const(NULL, 1, "auc_test");
 
 	osmo_init_logging2(tall_ctx, &hlr_log_info);
-	log_set_print_filename(osmo_stderr_target, cmdline_opts.verbose);
+	log_set_print_filename2(osmo_stderr_target,
+				cmdline_opts.verbose ?
+					LOG_FILENAME_BASENAME :
+					LOG_FILENAME_NONE);
 	log_set_print_timestamp(osmo_stderr_target, 0);
 	log_set_use_color(osmo_stderr_target, 0);
+	log_set_print_category_hex(osmo_stderr_target, 0);
 	log_set_print_category(osmo_stderr_target, 1);
 	log_parse_category_mask(osmo_stderr_target, "DMAIN,1:DDB,1:DAUC,1");
 
 	test_gen_vectors_2g_only();
 	test_gen_vectors_2g_plus_3g();
 	test_gen_vectors_3g_only();
+	test_gen_vectors_3g_xor();
 	test_gen_vectors_bad_args();
 
 	printf("Done\n");
